@@ -19,9 +19,13 @@ static std::unique_ptr<MupenWSIPlatform> wsi_platform;
 int32_t vk_rescaling;
 bool vk_ssreadbacks;
 bool vk_ssdither;
-unsigned vk_overscan;
-unsigned vk_vertical_stretch;
-unsigned vk_downscaling_steps;
+uint32_t vk_overscan_left;
+uint32_t vk_overscan_right;
+uint32_t vk_overscan_top;
+uint32_t vk_overscan_bottom;
+bool vk_overscan_enable;
+uint32_t vk_vertical_stretch;
+uint32_t vk_downscaling_steps;
 bool vk_native_texture_lod;
 bool vk_native_tex_rect;
 bool vk_divot_filter, vk_gamma_dither;
@@ -180,7 +184,11 @@ static void render_frame(Vulkan::Device &device)
 	opts.blend_previous_frame = vk_interlacing;
 	opts.upscale_deinterlacing = !vk_interlacing;
 	opts.downscale_steps = vk_downscaling_steps;
-	opts.crop_overscan_pixels = vk_overscan;
+	opts.crop_rect.left = vk_overscan_left;
+	opts.crop_rect.right = vk_overscan_right;
+	opts.crop_rect.top = vk_overscan_top;
+	opts.crop_rect.bottom = vk_overscan_bottom;
+	opts.crop_rect.enable = vk_overscan_enable;
 	if (vk_vertical_stretch) {
 		opts.crop_rect.top = vk_vertical_stretch;
 		opts.crop_rect.bottom = vk_vertical_stretch;
