@@ -156,6 +156,9 @@ static l_Setting get_setting(SettingsID settingId)
     case SettingsID::GUI_LastUpdateCheck:
         setting = {SETTING_SECTION_GUI, "LastUpdateCheck", std::string("")};
         break;
+    case SettingsID::GUI_ConfirmExitWhileInGame:
+        setting = {SETTING_SECTION_GUI, "ConfirmExitWhileInGame", true};
+        break;
     case SettingsID::GUI_DiscordRpc:
         setting = {SETTING_SECTION_GUI, "DiscordRpc", true};
         break;
@@ -167,7 +170,10 @@ static l_Setting get_setting(SettingsID settingId)
         setting = {SETTING_SECTION_NETPLAY, "Nickname", std::string("NetplayUser")};
         break;
     case SettingsID::Netplay_ServerJsonUrl:
-        setting = {SETTING_SECTION_NETPLAY, "ServerJsonUrl", std::string("https://m64p.s3.amazonaws.com/servers.json")};
+        setting = {SETTING_SECTION_NETPLAY, "ServerJsonUrl", std::string("")};
+        break;
+    case SettingsID::Netplay_DispatcherUrl:
+        setting = {SETTING_SECTION_NETPLAY, "DispatcherUrl", std::string("https://dispatch.gopher64.com")};
         break;
     case SettingsID::Netplay_SelectedServer:
         setting = {SETTING_SECTION_NETPLAY, "SelectedServer", std::string("")};
@@ -248,6 +254,9 @@ static l_Setting get_setting(SettingsID settingId)
     case SettingsID::Core_SaveFileNameFormat:
         setting = {SETTING_SECTION_M64P, "SaveFilenameFormat", 1};
         break;
+    case SettingsID::Core_GbCameraVideoCaptureBackend1:
+        setting = {SETTING_SECTION_M64P, "GbCameraVideoCaptureBackend1", std::string("sdl3")};
+        break;
 
     case SettingsID::CoreOverlay_RandomizeInterrupt:
         setting = {SETTING_SECTION_OVERLAY, "RandomizeInterrupt", true};
@@ -273,6 +282,9 @@ static l_Setting get_setting(SettingsID settingId)
     case SettingsID::CoreOverLay_SaveFileNameFormat:
         setting = {SETTING_SECTION_OVERLAY, "SaveFilenameFormat", 1};
         break;
+    case SettingsID::CoreOverlay_GbCameraVideoCaptureBackend1:
+        setting = {SETTING_SECTION_OVERLAY, "GbCameraVideoCaptureBackend1", std::string("sdl3")};
+        break;
 
     case SettingsID::Core_ScreenshotPath:
         setting = {SETTING_SECTION_M64P, "ScreenshotPath", CoreGetDefaultScreenshotDirectory().string(), "", true};
@@ -1678,6 +1690,10 @@ CORE_EXPORT bool CoreSettingsUpgrade(void)
     std::string settingsString;
 
     settingsVersion = CoreSettingsGetStringValue(SettingsID::GUI_Version);
+    if (settingsVersion.size() > 6)
+    { // strip git prefix
+        settingsVersion = settingsVersion.substr(0, 6);
+    }
 
     // we don't need to do anything
     // when the core version and the settings version match
@@ -1688,7 +1704,6 @@ CORE_EXPORT bool CoreSettingsUpgrade(void)
 
     if (settingsVersion.empty())
     { // settings version was introduced in >v0.1.5
-
 #ifndef PORTABLE_INSTALL // only applies to non-portable installs (i.e flatpak)
         // sadly v0.1.5 introduced an issue,
         // in v0.1.4 the screenshot directory was set to 'Screenshots',
@@ -1719,6 +1734,31 @@ CORE_EXPORT bool CoreSettingsUpgrade(void)
         }
     }
 
+    if (settingsVersion == "v0.6.8" || 
+        settingsVersion == "v0.6.9" ||
+        settingsVersion == "v0.7.0" ||
+        settingsVersion == "v0.7.0" ||
+        settingsVersion == "v0.7.1" ||
+        settingsVersion == "v0.7.2" ||
+        settingsVersion == "v0.7.3" ||
+        settingsVersion == "v0.7.4" ||
+        settingsVersion == "v0.7.5" ||
+        settingsVersion == "v0.7.6" ||
+        settingsVersion == "v0.7.7" ||
+        settingsVersion == "v0.7.8" ||
+        settingsVersion == "v0.7.9")
+    {
+        // in v0.8.0 RMG switched to the dispatcher server for netplay,
+        // to keep support for the json file, we only empty that setting
+        // when it is the default value because the default json url
+        // no longer exists
+        settingsString = CoreSettingsGetStringValue(SettingsID::Netplay_ServerJsonUrl);
+        if (settingsString == "https://m64p.s3.amazonaws.com/servers.json")
+        {
+            CoreSettingsSetValue(SettingsID::Netplay_ServerJsonUrl, std::string(""));
+        }
+    }
+
     // save core version
     CoreSettingsSetValue(SettingsID::GUI_Version, CoreGetVersion());
     CoreSettingsSave();
