@@ -39,12 +39,14 @@ enum class SettingsID
     GUI_IconTheme,
     GUI_CheckForUpdates,
     GUI_LastUpdateCheck,
+    GUI_ConfirmExitWhileInGame,
     GUI_DiscordRpc,
     GUI_Version,
 
     // Netplay Settings
     Netplay_Nickname,
     Netplay_ServerJsonUrl,
+    Netplay_DispatcherUrl,
     Netplay_SelectedServer,
 
     // Core Plugin Settings
@@ -89,6 +91,7 @@ enum class SettingsID
     Core_CountPerOpDenomPot,
     Core_SiDmaDuration,
     Core_SaveFileNameFormat,
+    Core_GbCameraVideoCaptureBackend1,
 
     // (mupen64plus) Overlay Core Settings
     CoreOverlay_RandomizeInterrupt,
@@ -99,6 +102,7 @@ enum class SettingsID
     CoreOverlay_CountPerOpDenomPot,
     CoreOverlay_SiDmaDuration,
     CoreOverLay_SaveFileNameFormat,
+    CoreOverlay_GbCameraVideoCaptureBackend1,
 
     // (mupen64plus) Core Directory Settings
     Core_ScreenshotPath,
