@@ -12,7 +12,6 @@
 #define INPUT_PLUGIN_API_VERSION 0x020100
 
 #include "UserInterface/MainDialog.hpp"
-#include "Utilities/InputDevice.hpp"
 #include "Thread/HotkeysThread.hpp"
 #include "Thread/SDLThread.hpp"
 #include "common.hpp"
@@ -40,7 +39,7 @@
 
 #include <QGuiApplication>
 #include <QApplication>
-#include <SDL.h>
+#include <SDL3/SDL.h>
 
 #include <algorithm>
 #include <chrono>
@@ -89,7 +88,7 @@ struct InputProfile
     std::string DeviceName;
     std::string DevicePath;
     std::string DeviceSerial;
-    int DeviceNum = -1;
+    InputDeviceType DeviceType = InputDeviceType::Invalid;
     std::chrono::time_point<std::chrono::high_resolution_clock> LastDeviceCheckTime = std::chrono::high_resolution_clock::now();
 
     // Gameboy information
@@ -97,7 +96,8 @@ struct InputProfile
     std::string GameboySave;
 
     // input device
-    Utilities::InputDevice InputDevice;
+    SDL_Joystick* SDLJoystick = nullptr;
+    SDL_Gamepad*  SDLGamepad  = nullptr;
 
     // buttons
     InputMapping Button_A;
@@ -204,7 +204,7 @@ static void (*l_DebugCallback)(void *, int, const char *) = nullptr;
 static void *l_DebugCallContext                           = nullptr;
 
 // keyboard state
-static bool l_KeyboardState[SDL_NUM_SCANCODES];
+static bool l_KeyboardState[SDL_SCANCODE_COUNT];
 
 // config GUI state
 static bool l_IsConfigGuiOpen = false;
@@ -309,23 +309,14 @@ static void load_settings(void)
 
         profile->PluggedIn = CoreSettingsGetBoolValue(SettingsID::Input_PluggedIn, section);
         profile->DeadzoneValue = CoreSettingsGetIntValue(SettingsID::Input_Deadzone, section);
-        profile->ControllerPak = (N64ControllerPak)CoreSettingsGetIntValue(SettingsID::Input_Pak, section);
+        profile->ControllerPak = static_cast<N64ControllerPak>(CoreSettingsGetIntValue(SettingsID::Input_Pak, section));
         profile->DeviceName = CoreSettingsGetStringValue(SettingsID::Input_DeviceName, section);
         profile->DevicePath = CoreSettingsGetStringValue(SettingsID::Input_DevicePath, section);
         profile->DeviceSerial = CoreSettingsGetStringValue(SettingsID::Input_DeviceSerial, section);
-        profile->DeviceNum = CoreSettingsGetIntValue(SettingsID::Input_DeviceNum, section);
+        profile->DeviceType = static_cast<InputDeviceType>(CoreSettingsGetIntValue(SettingsID::Input_DeviceType, section));
         profile->GameboyRom = CoreSettingsGetStringValue(SettingsID::Input_GameboyRom, section);
         profile->GameboySave = CoreSettingsGetStringValue(SettingsID::Input_GameboySave, section);
-
-        // keep compatibility with profiles before version v0.3.9
-        if (CoreSettingsKeyExists(section, "Sensitivity"))
-        {
-            profile->SensitivityValue = CoreSettingsGetIntValue(SettingsID::Input_Sensitivity, section);
-        }
-        else
-        {
-            profile->SensitivityValue = 100;
-        }
+        profile->SensitivityValue = CoreSettingsGetIntValue(SettingsID::Input_Sensitivity, section);
 
 #define LOAD_INPUT_MAPPING(mapping, setting) \
         load_inputmapping_settings(&profile->mapping, section, SettingsID::setting##_Name, SettingsID::setting##_InputType, SettingsID::setting##_Data, SettingsID::setting##_ExtraData)
@@ -407,7 +398,7 @@ static void apply_controller_profiles(void)
     {
         InputProfile* profile = &l_InputProfiles[i];
         int plugin = PLUGIN_NONE;
-        bool emulateVRU = (profile->DeviceNum == static_cast<int>(InputDeviceType::EmulateVRU));
+        bool emulateVRU = (profile->DeviceType == InputDeviceType::EmulateVRU);
 
         switch (profile->ControllerPak)
         {
@@ -448,6 +439,8 @@ static void apply_controller_profiles(void)
     }
 }
 
+static void controller_rumble_stop(InputProfile* profile); // forward declaration
+
 static void switch_controller_pak(InputProfile* profile, const int Control, const int pak)
 {
     const int currentPak = l_ControlInfo.Controls[Control].Plugin;
@@ -465,7 +458,7 @@ static void switch_controller_pak(InputProfile* profile, const int Control, cons
     // ensure that we stop the controller's rumble
     if (currentPak == PLUGIN_RAW)
     {
-        profile->InputDevice.StopRumble();
+        controller_rumble_stop(profile);
     }
 }
 
@@ -498,93 +491,232 @@ static void apply_gameboy_settings(void)
     CoreSettingsSave();
 }
 
-static void setup_device_automatic(int num, InputProfile* profile)
+static std::string string_from_const_char(const char* str)
 {
-    static int previousSdlDeviceNum = -1;
-
-    // reset variable when needed
-    if (num == 0)
+    std::string string;
+    if (str != nullptr)
     {
-        previousSdlDeviceNum = -1;
+        string = str;
     }
+    return string;
+}
 
-    bool foundSdlDevice                   = false;
-    SDL_GameController* sdlGameController = nullptr;
-    SDL_Joystick* sdlJoystick             = nullptr;
-    std::string sdlDeviceName;
+static void open_controller_automatic(int index, InputProfile* profile, SDL_JoystickID* joysticks, int joysticksCount)
+{
+    bool foundJoystick = false;
+    SDL_JoystickID joystickId;
+    std::string debugMessage;
 
-    for (int i = (previousSdlDeviceNum + 1); i < SDL_NumJoysticks(); i++)
+    std::string debugMessageBase;
+    debugMessageBase = "setup_device_automatic(";
+    debugMessageBase += std::to_string(index);
+    debugMessageBase += "): ";
+
+    for (int i = index; i < joysticksCount; i++)
     {
-        if (SDL_IsGameController(i))
+        joystickId = joysticks[i];
+
+        if (SDL_IsGamepad(joystickId))
         {
-            sdlGameController = SDL_GameControllerOpen(i);
-            if (sdlGameController != nullptr)
+            profile->SDLJoystick = SDL_OpenJoystick(joystickId);
+            profile->SDLGamepad = SDL_OpenGamepad(joystickId);
+            if (profile->SDLGamepad == nullptr)
             {
-                sdlDeviceName = SDL_GameControllerName(sdlGameController);
-                SDL_GameControllerClose(sdlGameController);
-                profile->DeviceNum   = i;
-                profile->DeviceName  = sdlDeviceName;
-                previousSdlDeviceNum = i;
-                foundSdlDevice       = true; 
-                break;
+                debugMessage = debugMessageBase + "SDL_OpenGamepad Failed: ";
+                debugMessage += SDL_GetError();
+                PluginDebugMessage(M64MSG_ERROR, debugMessage);
+                continue;
             }
+
+            debugMessage = debugMessageBase + "found gamepad: ";
+            debugMessage += string_from_const_char(SDL_GetGamepadName(profile->SDLGamepad));
+            PluginDebugMessage(M64MSG_VERBOSE, debugMessage);
+
+            foundJoystick = true;
+            break;
         }
         else
         {
-            sdlJoystick = SDL_JoystickOpen(i);
-            if (sdlJoystick != nullptr)
+            profile->SDLJoystick = SDL_OpenJoystick(joystickId);
+            profile->SDLGamepad = nullptr;
+            if (profile->SDLJoystick == nullptr)
             {
-                sdlDeviceName = SDL_JoystickName(sdlJoystick);
-                SDL_JoystickClose(sdlJoystick);
-                profile->DeviceNum   = i;
-                profile->DeviceName  = sdlDeviceName;
-                previousSdlDeviceNum = i;
-                foundSdlDevice       = true; 
-                break;
+                debugMessage = debugMessageBase + "SDL_OpenJoystick Failed: ";
+                debugMessage += SDL_GetError();
+                PluginDebugMessage(M64MSG_ERROR, debugMessage);
+                continue;
             }
+
+            debugMessage = debugMessageBase + "found joystick: ";
+            debugMessage += string_from_const_char(SDL_GetJoystickName(profile->SDLJoystick));
+            PluginDebugMessage(M64MSG_VERBOSE, debugMessage);
+
+            foundJoystick = true;
+            break;
         }
     }
 
-    if (!foundSdlDevice)
+    if (!foundJoystick)
     { // fallback to keyboard
-        if (num == 0)
+        if (index == 0)
         {
-            profile->DeviceNum = static_cast<int>(InputDeviceType::Keyboard);
+            debugMessage = debugMessageBase + "falling back to keyboard";
+            PluginDebugMessage(M64MSG_VERBOSE, debugMessage);
+
+            profile->DeviceType = InputDeviceType::Keyboard;
         }
         else
         {
+            debugMessage = debugMessageBase + "no device found";
+            PluginDebugMessage(M64MSG_VERBOSE, debugMessage);
+
             profile->PluggedIn = false;
 
             // only override present in core
             // when we have the control info
             if (l_HasControlInfo)
             {
-                l_ControlInfo.Controls[num].Present = 0;
+                l_ControlInfo.Controls[index].Present = 0;
             }
         }
     }
 }
 
+static void open_controller(InputProfile* profile, SDL_JoystickID* joysticks, int joysticksCount)
+{
+    SDL_JoystickID joystickId;
+    SDL_Joystick* joystick = nullptr;
+    SDL_Gamepad* gamepad = nullptr;
+    std::string errorMessage;
+
+    std::string deviceName;
+    std::string devicePath;
+    std::string deviceSerial;
+
+    for (int i = 0; i < joysticksCount; i++)
+    {
+        joystickId = joysticks[i];
+
+        if (SDL_IsGamepad(joystickId))
+        {
+            joystick = SDL_OpenJoystick(joystickId);
+            gamepad  = SDL_OpenGamepad(joystickId);
+            if (gamepad == nullptr)
+            {
+                errorMessage = "open_controller(): SDL_OpenGamepad Failed: ";
+                errorMessage += SDL_GetError();
+                PluginDebugMessage(M64MSG_ERROR, errorMessage);
+                continue;
+            }
+
+            deviceName = string_from_const_char(SDL_GetGamepadName(gamepad));
+            devicePath = string_from_const_char(SDL_GetGamepadPath(gamepad));
+            deviceSerial = string_from_const_char(SDL_GetGamepadSerial(gamepad));
+        }
+        else
+        {
+            gamepad  = nullptr;
+            joystick = SDL_OpenJoystick(joystickId);
+            if (joystick == nullptr)
+            {
+                errorMessage = "open_controller(): SDL_OpenJoystick Failed: ";
+                errorMessage += SDL_GetError();
+                PluginDebugMessage(M64MSG_ERROR, errorMessage);
+                continue;
+            }
+
+            deviceName = string_from_const_char(SDL_GetJoystickName(joystick));
+            devicePath = string_from_const_char(SDL_GetJoystickPath(joystick));
+            deviceSerial = string_from_const_char(SDL_GetJoystickSerial(joystick));
+        }
+
+        if (deviceName   == profile->DeviceName &&
+            devicePath   == profile->DevicePath &&
+            deviceSerial == profile->DeviceSerial)
+        {
+            profile->SDLJoystick = joystick;
+            profile->SDLGamepad = gamepad;
+            return;
+        }
+
+        if (joystick != nullptr)
+        {
+            SDL_CloseJoystick(joystick);
+            joystick = nullptr;
+        }
+        if (gamepad != nullptr)
+        {
+            SDL_CloseGamepad(gamepad);
+            gamepad = nullptr;
+        }
+    }
+}
+
+static void close_controller(InputProfile* profile)
+{
+    if (profile->SDLJoystick != nullptr)
+    {
+        SDL_CloseJoystick(profile->SDLJoystick);
+        profile->SDLJoystick = nullptr;
+    }
+    if (profile->SDLGamepad != nullptr)
+    {
+        SDL_CloseGamepad(profile->SDLGamepad);
+        profile->SDLGamepad = nullptr;
+    }
+}
+
+static void controller_rumble_start(InputProfile* profile)
+{
+    if (profile->SDLGamepad != nullptr)
+    {
+        SDL_RumbleGamepad(profile->SDLGamepad, 0xFFFF, 0xFFFF, SDL_HAPTIC_INFINITY);
+    }
+    else if (profile->SDLJoystick != nullptr)
+    {
+        SDL_RumbleJoystick(profile->SDLJoystick, 0xFFFF, 0xFFFF, SDL_HAPTIC_INFINITY);
+    }
+}
+
+static void controller_rumble_stop(InputProfile* profile)
+{
+    if (profile->SDLGamepad != nullptr)
+    {
+        SDL_RumbleGamepad(profile->SDLGamepad,  0, 0, 0);
+    }
+    else if (profile->SDLJoystick != nullptr)
+    {
+        SDL_RumbleJoystick(profile->SDLJoystick,  0, 0, 0);
+    }
+}
+
 static void open_controllers(void)
 {
     // force re-fresh joystick list
-    SDL_JoystickUpdate();
+    SDL_UpdateJoysticks();
+
+    int joysticksCount = 0;
+    SDL_JoystickID* joysticks = SDL_GetJoysticks(&joysticksCount);
 
     for (int i = 0; i < NUM_CONTROLLERS; i++)
     {
         InputProfile* profile = &l_InputProfiles[i];
 
-        profile->InputDevice.CloseDevice();
+        close_controller(profile);
 
-        if (profile->DeviceNum == static_cast<int>(InputDeviceType::Automatic))
+        if (profile->DeviceType == InputDeviceType::Automatic)
         {
-            setup_device_automatic(i, profile);
+            open_controller_automatic(i, profile, joysticks, joysticksCount);
         }
+        else if (profile->DeviceType == InputDeviceType::Joystick)
+        {
+            open_controller(profile, joysticks, joysticksCount);
+        }
+    }
 
-        if (profile->DeviceNum != static_cast<int>(InputDeviceType::Keyboard))
-        {
-            profile->InputDevice.OpenDevice(profile->DeviceName, profile->DevicePath, profile->DeviceSerial, profile->DeviceNum);
-        }
+    if (joysticks != nullptr)
+    {
+        SDL_free(joysticks);
     }
 }
 
@@ -594,7 +726,7 @@ static void close_controllers(void)
     {
         InputProfile* profile = &l_InputProfiles[i];
 
-        profile->InputDevice.CloseDevice();
+        close_controller(profile);
     }
 }
 
@@ -613,16 +745,16 @@ static int get_button_state(InputProfile* profile, const InputMapping* inputMapp
             {
                 if (allPressed && i > 0)
                 {
-                    state &= SDL_GameControllerGetButton(profile->InputDevice.GetGameControllerHandle(), (SDL_GameControllerButton)data);
+                    state &= SDL_GetGamepadButton(profile->SDLGamepad, (SDL_GamepadButton)data);
                 }
                 else
                 {
-                    state |= SDL_GameControllerGetButton(profile->InputDevice.GetGameControllerHandle(), (SDL_GameControllerButton)data);
+                    state |= SDL_GetGamepadButton(profile->SDLGamepad, (SDL_GamepadButton)data);
                 }
             } break;
             case InputType::GamepadAxis:
             {
-                int axis_value = SDL_GameControllerGetAxis(profile->InputDevice.GetGameControllerHandle(), (SDL_GameControllerAxis)data);
+                int axis_value = SDL_GetGamepadAxis(profile->SDLGamepad, (SDL_GamepadAxis)data);
                 if (allPressed && i > 0)
                 {
                     state &= (abs(axis_value) >= (SDL_AXIS_PEAK / 2) && (extraData ? axis_value > 0 : axis_value < 0)) ? 1 : 0;
@@ -636,27 +768,27 @@ static int get_button_state(InputProfile* profile, const InputMapping* inputMapp
             {
                 if (allPressed && i > 0)
                 {
-                    state &= SDL_JoystickGetButton(profile->InputDevice.GetJoystickHandle(), data);
+                    state &= SDL_GetJoystickButton(profile->SDLJoystick, data);
                 }
                 else
                 {
-                    state |= SDL_JoystickGetButton(profile->InputDevice.GetJoystickHandle(), data);
+                    state |= SDL_GetJoystickButton(profile->SDLJoystick, data);
                 }
             } break;
             case InputType::JoystickHat:
             {
                 if (allPressed && i > 0)
                 {
-                    state &= (SDL_JoystickGetHat(profile->InputDevice.GetJoystickHandle(), data) & extraData) ? 1 : 0;
+                    state &= (SDL_GetJoystickHat(profile->SDLJoystick, data) & extraData) ? 1 : 0;
                 }
                 else
                 {
-                    state |= (SDL_JoystickGetHat(profile->InputDevice.GetJoystickHandle(), data) & extraData) ? 1 : 0;
+                    state |= (SDL_GetJoystickHat(profile->SDLJoystick, data) & extraData) ? 1 : 0;
                 }
             } break;
             case InputType::JoystickAxis:
             {
-                int axis_value = SDL_JoystickGetAxis(profile->InputDevice.GetJoystickHandle(), data);
+                int axis_value = SDL_GetJoystickAxis(profile->SDLJoystick, data);
                 if (allPressed && i > 0)
                 {
                     state &= (abs(axis_value) >= (SDL_AXIS_PEAK / 2) && (extraData ? axis_value > 0 : axis_value < 0)) ? 1 : 0;
@@ -706,11 +838,11 @@ static double get_axis_state(InputProfile* profile, const InputMapping* inputMap
         {
             case InputType::GamepadButton:
             {
-                button_state |= SDL_GameControllerGetButton(profile->InputDevice.GetGameControllerHandle(), (SDL_GameControllerButton)data);
+                button_state |= SDL_GetGamepadButton(profile->SDLGamepad, (SDL_GamepadButton)data);
             } break;
             case InputType::GamepadAxis:
             {
-                double axis_value = SDL_GameControllerGetAxis(profile->InputDevice.GetGameControllerHandle(), (SDL_GameControllerAxis)data);
+                double axis_value = SDL_GetGamepadAxis(profile->SDLGamepad, (SDL_GamepadAxis)data);
                 if (axis_value < -32767.0) axis_value = -32767.0;
                 if (extraData ? axis_value > 0 : axis_value < 0)
                 {
@@ -721,15 +853,15 @@ static double get_axis_state(InputProfile* profile, const InputMapping* inputMap
             } break;
             case InputType::JoystickButton:
             {
-                button_state |= SDL_JoystickGetButton(profile->InputDevice.GetJoystickHandle(), data);
+                button_state |= SDL_GetJoystickButton(profile->SDLJoystick, data);
             } break;
             case InputType::JoystickHat:
             {
-                button_state |= (SDL_JoystickGetHat(profile->InputDevice.GetJoystickHandle(), data) & extraData) ? 1 : 0;
+                button_state |= (SDL_GetJoystickHat(profile->SDLJoystick, data) & extraData) ? 1 : 0;
             } break;
             case InputType::JoystickAxis:
             {
-                double axis_value = SDL_JoystickGetAxis(profile->InputDevice.GetJoystickHandle(), data);
+                double axis_value = SDL_GetJoystickAxis(profile->SDLJoystick, data);
                 if (axis_value < -32767.0) axis_value = -32767.0;
                 if (extraData ? axis_value > 0 : axis_value < 0)
                 {
@@ -866,7 +998,7 @@ static bool check_hotkeys(int Control)
 
     // we only have to check for hotkeys
     // when there's a controller opened
-    if (!profile->InputDevice.HasOpenDevice())
+    if (profile->SDLJoystick == nullptr)
     {
         return false;
     }
@@ -936,7 +1068,7 @@ static void sdl_init()
     std::string debugMessage;
     int ret = -1;
 
-    for (const int subsystem : {SDL_INIT_GAMECONTROLLER, SDL_INIT_AUDIO, SDL_INIT_VIDEO, SDL_INIT_HAPTIC})
+    for (const int subsystem : {SDL_INIT_GAMEPAD, SDL_INIT_HAPTIC})
     {
         if (!SDL_WasInit(subsystem))
         {
@@ -950,10 +1082,10 @@ static void sdl_init()
     // try to load SDL_GameControllerDB when the file exists
     if (std::filesystem::is_regular_file(gameControllerDbPath))
     {
-        ret = SDL_GameControllerAddMappingsFromFile(gameControllerDbPath.string().c_str());
+        ret = SDL_AddGamepadMappingsFromFile(gameControllerDbPath.string().c_str());
         if (ret == -1)
         {
-            debugMessage = "sdl_init(): SDL_GameControllerAddMappingsFromFile Failed: ";
+            debugMessage = "sdl_init(): SDL_AddGamepadMappingsFromFile Failed: ";
             debugMessage += SDL_GetError();
             PluginDebugMessage(M64MSG_WARNING, debugMessage);
         }
@@ -969,13 +1101,20 @@ static void sdl_init()
 
 static void sdl_quit()
 {
-    for (const int subsystem : {SDL_INIT_GAMECONTROLLER, SDL_INIT_HAPTIC})
+    for (const int subsystem : {SDL_INIT_GAMEPAD, SDL_INIT_HAPTIC})
     {
         if (SDL_WasInit(subsystem))
         {
             SDL_QuitSubSystem(subsystem);
         }
     }
+
+#ifdef VRU
+    if (HasVRUInitSDL() && SDL_WasInit(SDL_INIT_AUDIO))
+    {
+        SDL_QuitSubSystem(SDL_INIT_AUDIO);
+    }
+#endif
 }
 
 //
@@ -998,11 +1137,6 @@ EXPORT m64p_error CALL PluginStartup(m64p_dynlib_handle CoreLibHandle, void *Con
     l_SDLThread = new Thread::SDLThread(nullptr);
     l_SDLThread->start();
 
-    for (int i = 0; i < NUM_CONTROLLERS; i++)
-    {
-        l_InputProfiles[i].InputDevice.SetSDLThread(l_SDLThread);
-    }
-
     l_HotkeysThread = new Thread::HotkeysThread(check_hotkeys, nullptr);
     l_HotkeysThread->start();
 
@@ -1184,11 +1318,11 @@ EXPORT void CALL ControllerCommand(int Control, unsigned char* Command)
                 {
                     if (*data) 
                     {
-                        profile->InputDevice.StartRumble();
+                        controller_rumble_start(profile);
                     }
                     else
                     {
-                        profile->InputDevice.StopRumble();
+                        controller_rumble_stop(profile);
                     }
                 }
                 data[32] = data_crc( data, 32 );
@@ -1213,7 +1347,7 @@ EXPORT void CALL GetKeys(int Control, BUTTONS* Keys)
 #ifdef VRU
     // when we're emulating the VRU,
     // we need to check the mic state
-    if (profile->DeviceNum == static_cast<int>(InputDeviceType::EmulateVRU))
+    if (profile->DeviceType == InputDeviceType::EmulateVRU)
     {
         if (GetVRUMicState())
         {
@@ -1227,31 +1361,6 @@ EXPORT void CALL GetKeys(int Control, BUTTONS* Keys)
     }
 #endif // VRU
 
-#if 0 // TODO: fix hotplug support
-    // check if device has been disconnected,
-    // if it has, try to open it again,
-    // only do this every 2 seconds to prevent lag
-    const auto currentTime  = std::chrono::high_resolution_clock::now();
-    const int secondsPassed = std::chrono::duration_cast<std::chrono::seconds>(currentTime - profile->LastDeviceCheckTime).count();
-    if (secondsPassed >= 2)
-    {
-        profile->LastDeviceCheckTime = currentTime;
-
-        if (profile->DeviceNum != static_cast<int>(InputDeviceType::Keyboard))
-        {
-            if (profile->InputDevice.IsOpeningDevice())
-            {
-                return;
-            }
-
-            if (!profile->InputDevice.HasOpenDevice() || !profile->InputDevice.IsAttached())
-            {
-                profile->InputDevice.OpenDevice(profile->DeviceName, profile->DevicePath, profile->DeviceSerial, profile->DeviceNum);
-            }
-        }
-    }
-#endif
-
     // when we've matched a hotkey,
     // we don't need to check anything
     // else
@@ -1309,7 +1418,7 @@ EXPORT void CALL GetKeys(int Control, BUTTONS* Keys)
 
 EXPORT void CALL InitiateControllers(CONTROL_INFO ControlInfo)
 {
-    for (int i = 0; i < SDL_NUM_SCANCODES; i++)
+    for (int i = 0; i < SDL_SCANCODE_COUNT; i++)
     {
         l_KeyboardState[i] = 0;
     }
