@@ -21,6 +21,7 @@
 #include <QFileDialog>
 #include <QJsonObject>
 #include <QJsonArray>
+#include <QUrlQuery>
 #include <QFile>
 
 #include <RMG-Core/Settings.hpp>
@@ -70,6 +71,11 @@ NetplaySessionBrowserDialog::NetplaySessionBrowserDialog(QWidget *parent, QWebSo
     this->nickNameLineEdit->setValidator(new QRegularExpressionValidator(re, this));
     this->nickNameLineEdit->setText(QString::fromStdString(CoreSettingsGetStringValue(SettingsID::Netplay_Nickname)));
 
+    // configure dispatcher network access manager
+    this->dispatcherNetworkAccessManager = new QNetworkAccessManager(this);
+    this->dispatcherNetworkAccessManager->setTransferTimeout(15000);
+    connect(this->dispatcherNetworkAccessManager, &QNetworkAccessManager::finished, this, &NetplaySessionBrowserDialog::on_dispatcherRetrieveServers_Finished);
+
     // request server list
     QString serverUrl = QString::fromStdString(CoreSettingsGetStringValue(SettingsID::Netplay_ServerJsonUrl));
     if (!serverUrl.isEmpty())
@@ -90,15 +96,28 @@ NetplaySessionBrowserDialog::NetplaySessionBrowserDialog(QWidget *parent, QWebSo
         else if (QUrl(serverUrl).isValid())
         {
             QNetworkAccessManager* networkAccessManager = new QNetworkAccessManager(this);
-            connect(networkAccessManager, &QNetworkAccessManager::finished, this, &NetplaySessionBrowserDialog::on_networkAccessManager_Finished);
+            connect(networkAccessManager, &QNetworkAccessManager::finished, this, &NetplaySessionBrowserDialog::on_jsonServerListDownload_Finished);
             networkAccessManager->setTransferTimeout(15000);
             networkAccessManager->get(QNetworkRequest(QUrl(serverUrl)));
         }
     }
 
-    this->validateJoinButton();
+    QString dispatcherUrl = QString::fromStdString(CoreSettingsGetStringValue(SettingsID::Netplay_DispatcherUrl));
+    if (!dispatcherUrl.isEmpty() && QUrl(dispatcherUrl).isValid())
+    {
+        this->dispatcherUrl = dispatcherUrl;
 
-    this->pingTimerId = this->startTimer(2000);
+        QNetworkAccessManager* networkAccessManager = new QNetworkAccessManager(this);
+        connect(networkAccessManager, &QNetworkAccessManager::finished, this, &NetplaySessionBrowserDialog::on_dispatcherRegionListDownload_Finished);
+        networkAccessManager->setTransferTimeout(15000);
+
+        QNetworkRequest networkRequest(QUrl(this->dispatcherUrl + "/getRegions"));
+        networkRequest.setRawHeader("netplay-id", "RMG");
+
+        networkAccessManager->get(networkRequest);
+    }
+
+    this->validateJoinButton();
 }
 
 NetplaySessionBrowserDialog::~NetplaySessionBrowserDialog(void)
@@ -179,6 +198,79 @@ void NetplaySessionBrowserDialog::validateJoinButton(void)
     joinButton->setEnabled(this->validate());
 }
 
+void NetplaySessionBrowserDialog::toggleUI(bool enable, bool enableJoinButton)
+{
+    QPushButton* joinButton = this->buttonBox->button(QDialogButtonBox::Ok);
+    joinButton->setEnabled(enableJoinButton);
+
+    QPushButton* refreshButton = this->buttonBox->button(QDialogButtonBox::RestoreDefaults);
+    refreshButton->setEnabled(enable);
+
+    this->serverComboBox->setEnabled(enable);
+    this->nickNameLineEdit->setReadOnly(!enable);
+}
+
+void NetplaySessionBrowserDialog::refreshSessions(void)
+{
+    // disable refresh button while refreshing
+    QPushButton* refreshButton = this->buttonBox->button(QDialogButtonBox::RestoreDefaults);
+    refreshButton->setEnabled(false);
+
+    // disable join button while refreshing
+    QPushButton* joinButton = this->buttonBox->button(QDialogButtonBox::Ok);
+    joinButton->setEnabled(false);
+
+    if (NetplayCommon::IsServerDispatcher(this->serverComboBox))
+    {
+        this->on_serverComboBox_currentIndexChanged(this->serverComboBox->currentIndex());
+    }
+    else
+    {
+        this->on_webSocket_connected();
+    }
+}
+
+void NetplaySessionBrowserDialog::joinSession(void)
+{
+    QJsonObject json;
+    QJsonObject session;
+    session.insert("port", sessionData.Port);
+    session.insert("password", this->sessionPassword);
+    session.insert("MD5", this->sessionData.MD5);
+
+    json.insert("type", "request_join_room");
+    json.insert("player_name", this->nickNameLineEdit->text());
+    json.insert("room", session);
+    NetplayCommon::AddCommonJson(json);
+
+    this->webSocket->sendTextMessage(QJsonDocument(json).toJson());
+}
+
+void NetplaySessionBrowserDialog::resetDispatcherState(void)
+{
+    if (this->dispatcherTimerId != -1)
+    {
+        this->killTimer(this->dispatcherTimerId);
+        this->dispatcherTimerId = -1;
+    }
+
+    if (this->dispatcherTimeoutTimerId != -1)
+    {
+        this->killTimer(this->dispatcherTimeoutTimerId);
+        this->dispatcherTimeoutTimerId = -1;
+    }
+
+    if (this->dispatcherNetworkReply != nullptr)
+    {
+        this->dispatcherNetworkReply->abort();
+        this->dispatcherNetworkReply = nullptr;
+    }
+
+    this->dispatcherMoveThroughList  = false;
+    this->dispatcherJoinSession      = false;
+    this->dispatcherAddressListIndex = false;
+}
+
 void NetplaySessionBrowserDialog::timerEvent(QTimerEvent *event)
 {
     if (event->timerId() == this->pingTimerId)
@@ -188,6 +280,44 @@ void NetplaySessionBrowserDialog::timerEvent(QTimerEvent *event)
             this->webSocket->ping();
         }
     }
+    else if (event->timerId() == this->dispatcherTimerId)
+    {
+        if (this->dispatcherMoveThroughList)
+        {
+            this->dispatcherAddressListIndex++;
+
+            if (this->dispatcherAddressListIndex < this->dispatcherAddressList.size())
+            {
+                QString address = this->dispatcherAddressList.at(this->dispatcherAddressListIndex);
+                this->webSocket->open(QUrl(address));
+
+                this->dispatcherMoveThroughList = false;
+
+                // start socket timeout
+                if (this->dispatcherTimeoutTimerId != -1)
+                {
+                    this->killTimer(this->dispatcherTimeoutTimerId);
+                    this->dispatcherTimeoutTimerId = -1;
+                }
+
+                this->dispatcherTimeoutTimerId = this->startTimer(15000);
+            }
+            else
+            {
+                this->sessionBrowserWidget->RefreshDone();
+                this->resetDispatcherState();
+            }
+        }
+    }
+    else if (event->timerId() == this->dispatcherTimeoutTimerId)
+    {
+        this->webSocket->close();
+        this->dispatcherMoveThroughList = true;
+
+        this->killTimer(this->dispatcherTimeoutTimerId);
+        this->dispatcherTimeoutTimerId = -1;
+
+    }
 }
 
 void NetplaySessionBrowserDialog::on_webSocket_connected(void)
@@ -198,16 +328,20 @@ void NetplaySessionBrowserDialog::on_webSocket_connected(void)
         return;
     }
 
-    // disable refresh button while refreshing
-    QPushButton* refreshButton = this->buttonBox->button(QDialogButtonBox::RestoreDefaults);
-    refreshButton->setEnabled(false);
+    bool dispatcher = NetplayCommon::IsServerDispatcher(this->serverComboBox);
 
-    // disable join button while refreshing
-    QPushButton* joinButton = this->buttonBox->button(QDialogButtonBox::Ok);
-    joinButton->setEnabled(false);
+    if (dispatcher && this->dispatcherJoinSession)
+    {
+        this->joinSession();
+        this->dispatcherJoinSession = false;
+        return;
+    }
 
-    // clear sessions
-    this->sessionBrowserWidget->StartRefresh();
+    // clear sessions when we're not using the dispatcher server
+    if (!dispatcher)
+    {
+        this->sessionBrowserWidget->StartRefresh();
+    }
 
     // request session list from server
     QJsonObject json;
@@ -224,6 +358,8 @@ void NetplaySessionBrowserDialog::on_webSocket_textMessageReceived(QString messa
 
     QString type = json.value("type").toString();
 
+    bool dispatcher = NetplayCommon::IsServerDispatcher(this->serverComboBox);
+
     if (type == "reply_get_rooms")
     {
         if (json.value("accept").toInt() == 0)
@@ -241,11 +377,15 @@ void NetplaySessionBrowserDialog::on_webSocket_textMessageReceived(QString messa
                                                             session.value("port").toInt(),
                                                             session.value("features").toObject().value("cpu_emulator").toString(),
                                                             session.value("features").toObject().value("rsp_plugin").toString(),
-                                                            session.value("features").toObject().value("gfx_plugin").toString());
+                                                            session.value("features").toObject().value("gfx_plugin").toString(),
+                                                            this->webSocket->requestUrl());
             }
 
             // we're done refreshing the sessions
-            this->sessionBrowserWidget->RefreshDone();
+            if (!dispatcher)
+            {
+                this->sessionBrowserWidget->RefreshDone();
+            }
         }
         else
         {
@@ -263,19 +403,30 @@ void NetplaySessionBrowserDialog::on_webSocket_textMessageReceived(QString messa
         else
         {
             QtMessageBox::Error(this, "Server Error", json.value("message").toString());
-            this->validateJoinButton();
+            this->toggleUI(true, this->validate());
         }
     }
+
+    if (dispatcher)
+    {
+        this->dispatcherMoveThroughList = true;
+    }
 }
 
 void NetplaySessionBrowserDialog::on_webSocket_pong(quint64 elapsedTime, const QByteArray&)
 {
-    this->pingLineEdit->setText(QString::number(elapsedTime) + " ms");
+    if (!NetplayCommon::IsServerDispatcher(this->serverComboBox))
+    {
+        this->pingLineEdit->setText(QString::number(elapsedTime) + " ms");
+    }
 }
 
 void NetplaySessionBrowserDialog::on_webSocket_disconnected()
 {
-    this->sessionBrowserWidget->Reset();
+    if (!NetplayCommon::IsServerDispatcher(this->serverComboBox))
+    {
+        this->sessionBrowserWidget->Reset();
+    }
 }
 
 void NetplaySessionBrowserDialog::on_broadcastSocket_readyRead(void)
@@ -284,20 +435,14 @@ void NetplaySessionBrowserDialog::on_broadcastSocket_readyRead(void)
     {
         QNetworkDatagram datagram = this->broadcastSocket.receiveDatagram();
         QByteArray incomingData = datagram.data();
-        QJsonDocument json_doc  = QJsonDocument::fromJson(incomingData);
-        QJsonObject json        = json_doc.object();
-        QStringList servers     = json.keys();
 
-        for (int i = 0; i < servers.size(); i++)
-        {
-            this->serverComboBox->addItem(servers.at(i), json.value(servers.at(i)).toString());
-        }
+        NetplayCommon::AddServers(this->serverComboBox, QJsonDocument::fromJson(incomingData));
     }
 
     NetplayCommon::RestoreSelectedServer(this->serverComboBox);
 }
 
-void NetplaySessionBrowserDialog::on_networkAccessManager_Finished(QNetworkReply* reply)
+void NetplaySessionBrowserDialog::on_jsonServerListDownload_Finished(QNetworkReply* reply)
 {
     if (reply->error())
     {
@@ -313,6 +458,61 @@ void NetplaySessionBrowserDialog::on_networkAccessManager_Finished(QNetworkReply
     reply->deleteLater();
 }
 
+void NetplaySessionBrowserDialog::on_dispatcherRegionListDownload_Finished(QNetworkReply* reply)
+{
+    if (reply->error())
+    {
+        this->sessionBrowserWidget->Reset();
+        QtMessageBox::Error(this, "Server Error", "Failed to retrieve region list: " + reply->errorString());
+        reply->deleteLater();
+        return;
+    }
+
+    NetplayCommon::AddServers(this->serverComboBox, 
+                              QJsonDocument::fromJson(reply->readAll()), true);
+
+    reply->deleteLater();
+}
+
+void NetplaySessionBrowserDialog::on_dispatcherRetrieveServers_Finished(QNetworkReply* reply)
+{
+    // don't show an error when the request has been aborted
+    if (reply->error() == QNetworkReply::OperationCanceledError)
+    {
+        reply->deleteLater();
+        return;
+    }
+
+    if (reply->error())
+    {
+        this->sessionBrowserWidget->Reset();
+        QtMessageBox::Error(this, "Server Error", "Failed to retrieve server list: " + reply->errorString());
+        reply->deleteLater();
+        return;
+    }
+
+    QJsonDocument jsonDocument = QJsonDocument::fromJson(reply->readAll());
+    QJsonObject jsonObject = jsonDocument.object();
+    QStringList jsonKeys = jsonObject.keys();
+
+    this->sessionBrowserWidget->StartRefresh();
+    this->resetDispatcherState();
+
+    // we only care about the addresses
+    this->dispatcherAddressList.clear();
+    for (int i = 0; i < jsonKeys.size(); i++)
+    {
+        QString address = jsonObject[jsonKeys.at(i)].toString();
+        this->dispatcherAddressList.append(address);
+    }
+
+    this->dispatcherAddressListIndex = -1;
+    this->dispatcherMoveThroughList = true;
+    this->dispatcherTimerId = this->startTimer(250);
+
+    reply->deleteLater();
+}
+
 void NetplaySessionBrowserDialog::on_serverComboBox_currentIndexChanged(int index)
 {
     QPushButton* refreshButton = this->buttonBox->button(QDialogButtonBox::RestoreDefaults);
@@ -323,12 +523,48 @@ void NetplaySessionBrowserDialog::on_serverComboBox_currentIndexChanged(int inde
         return;
     }
 
-    this->pingLineEdit->setText("Calculating...");
+    bool dispatcher = NetplayCommon::IsServerDispatcher(this->serverComboBox, index);
 
+    if (this->pingTimerId != -1)
+    {
+        this->killTimer(this->pingTimerId);
+        this->pingTimerId = -1;
+    }
+
+    this->resetDispatcherState();
+    this->pingLineEdit->setText(dispatcher ? "N/A" : "Calculating...");
     this->sessionBrowserWidget->StartRefresh();
 
-    QString address = this->serverComboBox->itemData(index).toString();
-    this->webSocket->open(QUrl(address));
+    if (dispatcher)
+    {
+        QString region = NetplayCommon::GetServerData(this->serverComboBox, index);
+
+        QUrl url(this->dispatcherUrl + "/getServers");
+
+        QUrlQuery urlQuery;
+        urlQuery.addQueryItem("region", region);
+        url.setQuery(urlQuery);
+
+        QNetworkRequest networkRequest(url);
+        networkRequest.setRawHeader("netplay-id", "RMG");
+
+        // sadly we have to force HTTP/1 here due to a Qt bug,
+        // we abort the connection which sadly in Qt6 causes an error and eventually segfault,
+        // so to workaround this, force HTTP/1 until it's fixed in Qt6
+        //
+        // thank you MapLibre devs for figuring out this issue:
+        // https://github.com/maplibre/maplibre-native/issues/3644
+        networkRequest.setAttribute(QNetworkRequest::Http2AllowedAttribute, false);
+
+        this->dispatcherNetworkReply = this->dispatcherNetworkAccessManager->get(networkRequest);
+    }
+    else
+    {
+        this->pingTimerId = this->startTimer(2000);
+
+        QString address = NetplayCommon::GetServerData(this->serverComboBox, index);
+        this->webSocket->open(QUrl(address));
+    }
 }
 
 void NetplaySessionBrowserDialog::on_sessionBrowserWidget_OnSessionChanged(bool valid)
@@ -353,13 +589,14 @@ void NetplaySessionBrowserDialog::on_buttonBox_clicked(QAbstractButton* button)
     // refresh session list when refresh button has been pressed
     if (button == this->buttonBox->button(QDialogButtonBox::RestoreDefaults))
     {
-        this->on_webSocket_connected();
+        this->refreshSessions();
     }
 }
 
 void NetplaySessionBrowserDialog::accept()
 {
-    if (!this->webSocket->isValid())
+    if (!NetplayCommon::IsServerDispatcher(this->serverComboBox) &&
+        !this->webSocket->isValid())
     {
         QtMessageBox::Error(this, "Server Error", "Connection Failed");
         return;
@@ -373,21 +610,19 @@ void NetplaySessionBrowserDialog::accept()
     }
 
     // disable join button while we're processing the request
-    QPushButton* joinButton = this->buttonBox->button(QDialogButtonBox::Ok);
-    joinButton->setEnabled(false);
+    this->toggleUI(false, false);
 
     // request password when needed
-    QString password;
     if (sessionData.PasswordProtected)
     {
         NetplaySessionPasswordDialog dialog(this);
         dialog.exec();
-        password = dialog.GetPassword();
+        this->sessionPassword = dialog.GetPassword();
 
         // do nothing if password is empty
-        if (password.isEmpty())
+        if (this->sessionPassword.isEmpty())
         {
-            joinButton->setEnabled(true);
+            this->toggleUI(true, true);
             return;
         }
     }
@@ -400,7 +635,7 @@ void NetplaySessionBrowserDialog::accept()
         details  = "Expected RSP Plugin: " + sessionData.RspPlugin + "\n";
         details += "Current  RSP Plugin: " + pluginNames[0];
         QtMessageBox::Error(this, "RSP Plugin Mismatch", details);
-        joinButton->setEnabled(true);
+        this->toggleUI(true, true);
         return;
     }
     if (sessionData.GfxPlugin != pluginNames[1])
@@ -408,7 +643,7 @@ void NetplaySessionBrowserDialog::accept()
         details  = "Expected GFX Plugin: " + sessionData.GfxPlugin + "\n";
         details += "Current  GFX Plugin: " + pluginNames[1];
         QtMessageBox::Error(this, "GFX Plugin Mismatch", details);
-        joinButton->setEnabled(true);
+        this->toggleUI(true, true);
         return;
     }
 
@@ -430,21 +665,20 @@ void NetplaySessionBrowserDialog::accept()
         this->sessionFile = this->showROMDialog(sessionData.GameName, sessionData.MD5);
         if (this->sessionFile.isEmpty())
         {
-            joinButton->setEnabled(true);
+            this->toggleUI(true, true);
             return;
         }
     }
 
-    QJsonObject json;
-    QJsonObject session;
-    session.insert("port", sessionData.Port);
-    session.insert("password", password);
-    session.insert("MD5", QString::fromStdString(md5));
+    this->sessionData = sessionData;
 
-    json.insert("type", "request_join_room");
-    json.insert("player_name", this->nickNameLineEdit->text());
-    json.insert("room", session);
-    NetplayCommon::AddCommonJson(json);
-
-    this->webSocket->sendTextMessage(QJsonDocument(json).toJson());
+    if (NetplayCommon::IsServerDispatcher(this->serverComboBox))
+    {
+        this->dispatcherJoinSession = true;
+        this->webSocket->open(sessionData.Address);
+    }
+    else
+    {
+        this->joinSession();
+    }
 }
