@@ -24,7 +24,7 @@
 #include "Dialog/Netplay/NetplaySessionDialog.hpp"
 #endif // NETPLAY
 #include "UserInterface/EventFilter.hpp"
-#include "Utilities/QtKeyToSdl2Key.hpp"
+#include "Utilities/QtKeyToSdl3Key.hpp"
 #include "Utilities/QtMessageBox.hpp"
 #include "OnScreenDisplay.hpp"
 #include "Callbacks.hpp"
@@ -49,6 +49,7 @@
 #include <QSettings>
 #include <QStatusBar>
 #include <QMenuBar>
+#include <cstdlib>
 #include <QString>
 #include <QTimer>
 #include <cmath>
@@ -162,11 +163,27 @@ void MainWindow::OpenROM(QString file, QString disk, bool fullscreen, bool quitA
 
 void MainWindow::closeEvent(QCloseEvent *event)
 {
+    bool inEmulation = this->emulationThread->isRunning();
+
+    if (!this->ui_ForceClose &&
+        inEmulation &&
+        CoreSettingsGetBoolValue(SettingsID::GUI_ConfirmExitWhileInGame))
+    {
+        bool skipExitConfirmation = false;
+        bool ret = QtMessageBox::Question(this, "Are you sure you want to exit RMG?", "Don't ask for confirmation again", skipExitConfirmation);
+        CoreSettingsSetValue(SettingsID::GUI_ConfirmExitWhileInGame, !skipExitConfirmation);
+        if (!ret)
+        {
+            event->ignore();
+            return;
+        }
+    }
+
     // we have to make sure we save the geomtry
     // for the ROM browser when emulation
     // isn't running (or hasn't run at all)
     if (!this->ui_QuitAfterEmulation && 
-        !this->emulationThread->isRunning())
+        !inEmulation)
     {
         this->storeGeometry();
     }
@@ -306,11 +323,7 @@ void MainWindow::configureUI(QApplication* app, bool showUI)
     this->installEventFilter(this->ui_EventFilter);
     this->ui_Widget_Dummy->installEventFilter(this->ui_EventFilter);
 
-    this->ui_WindowTitle = QCoreApplication::applicationName();
-    this->ui_WindowTitle += " (";
-    this->ui_WindowTitle += QString::fromStdString(CoreGetVersion());
-    this->ui_WindowTitle += ")";
-
+    this->ui_WindowTitle = this->getWindowTitle();
     this->setWindowTitle(this->ui_WindowTitle);
 }
 
@@ -330,6 +343,12 @@ void MainWindow::configureTheme(QApplication* app)
     {
         // do nothing
     }
+#ifdef _WIN32
+    else if (theme == "Windows Vista")
+    {
+        app->setStyle(QStyleFactory::create("WindowsVista"));
+    }
+#endif
     else if (theme == "Fusion")
     {
         app->setPalette(QApplication::style()->standardPalette());
@@ -413,6 +432,51 @@ void MainWindow::configureTheme(QApplication* app)
     QIcon::setFallbackThemeName(fallbackThemeName);
 }
 
+QString MainWindow::getWindowTitle(void)
+{
+    const QDate currentDate = QDateTime::currentDateTime().date();
+    const QStringList firstWordList = {
+    {
+        "lesbian",
+        "gay",
+        "bisexual",
+        "transgender",
+        "queer",
+    }};
+    const QStringList secondWordList = {
+    {
+        " rights!!!",
+        "s rise up!!!"
+    }};
+
+    QString windowTitle = QCoreApplication::applicationName();
+
+    // initialize random seed
+    srand(time(nullptr));
+
+    bool showCustomWindowTitle = (rand() % 10) < 3;
+
+    if (showCustomWindowTitle && currentDate.month() == 3 && currentDate.day() == 31)
+    {
+        QString secondWord = secondWordList.at(rand() % secondWordList.count());
+        windowTitle += " (transgender" + secondWord + ")";
+    }
+    else if (showCustomWindowTitle && currentDate.month() == 6)
+    {
+        QString firstWsord = firstWordList.at(rand() % firstWordList.count());
+        QString secondWord = secondWordList.at(rand() % secondWordList.count());
+        windowTitle += " (" + firstWsord + secondWord + ")";
+    }
+    else
+    {
+        windowTitle += " (";
+        windowTitle += QString::fromStdString(CoreGetVersion());
+        windowTitle += ")";
+    }
+
+    return windowTitle;
+}
+
 void MainWindow::showErrorMessage(QString text, QString details, bool force)
 {
     // fallback to helper when forced
@@ -658,7 +722,7 @@ void MainWindow::launchEmulationThread(QString cartRom, QString address, int por
     this->launchEmulationThread(cartRom, "", false, -1, true);
 }
 
-void MainWindow::launchEmulationThread(QString cartRom, QString diskRom, bool refreshRomListAfterEmulation, int slot, bool netplay)
+void MainWindow::launchEmulationThread(QString cartRom, QString diskRom, bool refreshRomListAfterEmulation, int slot, bool netplay, bool dragdrop)
 {
 #ifdef NETPLAY
     if (this->netplaySessionDialog != nullptr && !netplay)
@@ -668,6 +732,12 @@ void MainWindow::launchEmulationThread(QString cartRom, QString diskRom, bool re
     }
 #endif // NETPLAY
 
+    if (!dragdrop && this->emulationThread->isRunning())
+    {
+        this->showErrorMessage("EmulationThread::run Failed", "Cannot start emulation when emulation is already running or being started");
+        return;
+    }
+
     CoreSettingsSave();
 
     if (this->emulationThread->isRunning())
@@ -1371,8 +1441,8 @@ void MainWindow::on_EventFilter_KeyPressed(QKeyEvent *event)
         return;
     }
 
-    int key = Utilities::QtKeyToSdl2Key(event->key());
-    int mod = Utilities::QtModKeyToSdl2ModKey(event->modifiers());
+    int key = Utilities::QtKeyToSdl3Key(event->key());
+    int mod = Utilities::QtModKeyToSdl3ModKey(event->modifiers());
 
     CoreSetKeyDown(key, mod);
 }
@@ -1385,8 +1455,8 @@ void MainWindow::on_EventFilter_KeyReleased(QKeyEvent *event)
         return;
     }
 
-    int key = Utilities::QtKeyToSdl2Key(event->key());
-    int mod = Utilities::QtModKeyToSdl2ModKey(event->modifiers());
+    int key = Utilities::QtKeyToSdl3Key(event->key());
+    int mod = Utilities::QtModKeyToSdl3ModKey(event->modifiers());
 
     CoreSetKeyUp(key, mod);
 }
@@ -1407,6 +1477,13 @@ void MainWindow::on_EventFilter_FileDropped(QDropEvent *event)
     bool refreshRomList  = false;
     QString file;
 
+    // when we're still opening the ROM while emulation is running,
+    // ignore the event
+    if (this->emulationThread->isRunning() && !CoreHasRomOpen())
+    {
+        return;
+    }
+
     if (inEmulation && confirmDragDrop)
     {
         QMessageBox::StandardButton reply = QMessageBox::question(this, "",
@@ -1432,7 +1509,7 @@ void MainWindow::on_EventFilter_FileDropped(QDropEvent *event)
         refreshRomList = this->ui_RefreshRomListAfterEmulation;
     }
 
-    this->launchEmulationThread(file, "", refreshRomList);
+    this->launchEmulationThread(file, "", refreshRomList, -1, false, true);
 #endif // DRAG_DROP
 }
 
@@ -1516,7 +1593,8 @@ void MainWindow::on_networkAccessManager_Finished(QNetworkReply* reply)
     }
 
 #ifdef APPIMAGE_UPDATER
-    this->on_Action_System_Exit();
+    this->ui_ForceClose = true;
+    this->close();
 #else // normal updater
     Dialog::InstallUpdateDialog installUpdateDialog(this, QCoreApplication::applicationDirPath(), downloadUpdateDialog.GetTempDirectory(), downloadUpdateDialog.GetFileName());
     ret = installUpdateDialog.exec();
@@ -2096,6 +2174,8 @@ void MainWindow::on_Emulation_Finished(bool ret, QString error)
         {
             this->showErrorMessage("EmulationThread::run Failed", error);
         }
+
+        this->ui_ForceClose = true;
         this->close();
         return;
     }
